local FOLDER_NAME, private = ...
local core = LibStub("AceAddon-3.0"):NewAddon(FOLDER_NAME, "AceTimer-3.0", "AceEvent-3.0")
local L = LibStub:GetLibrary("AceLocale-3.0"):GetLocale(FOLDER_NAME)
local strformat, pairs, strupper, select, type = string.format, pairs, strupper, select, type
local UnitName, UnitClass, GetItemCount, GetItemIcon, GetNumGuildBankTabs, GetGuildBankTabInfo, GetGuildBankItemInfo, GetGuildBankItemLink, IsInGuild = UnitName, UnitClass, GetItemCount, GetItemIcon, GetNumGuildBankTabs, GetGuildBankTabInfo, GetGuildBankItemInfo, GetGuildBankItemLink, IsInGuild
local oldTime, scanPending
local charsList = {}
local playerName, playerClass = UnitName("player"), strupper(select(2,UnitClass("player")))
local MAX_GUILDBANK_SLOTS_PER_TAB = 98

private.scanPending = scanPending

local defaults = {
    profile = {
        modules = {['*'] = false},
        tracking = {['*'] = false},
        showShort = false,
        showIcons = false,
        iconsOnly = false,
        tooltipIcons = false,
        tooltipIconsOnly = false,
        tooltipShowShort = false,
        lineColor = true,
        textColor = true,
        showZero = true,
        showBank = true,
        showAlts = true,
        showGuild = true,
        updatetime = 5,
        scantime = 5,
        scale = 1,
        scroll = 900,
    },
    realm = {
        chars = {
            ['*'] = {
                data = {
                    ['*'] = {inv = 0, bank = 0, guild = 0},
                },
            }
        }
    }
}

function core:OnInitialize()
    self.charName = playerName
    self.db = LibStub("AceDB-3.0"):New(FOLDER_NAME.."_DB", defaults, true)
    if not self.db.realm.chars[self.charName].class then
        self.db.realm.chars[self.charName].class = playerClass
    end
    for name in pairs(self.db.realm.chars) do
            charsList[name] = name
    end
    oldTime = self.db.profile.updatetime
    self:SetupOptions()
end

private.charsList = charsList

function core:OnEnable()
    self.db.RegisterCallback(self, "OnProfileChanged", "applySettings")
    self.db.RegisterCallback(self, "OnProfileCopied", "applySettings")
    self.db.RegisterCallback(self, "OnProfileReset", "applySettings")
    if not self.updateTimer then
        self:setTimer()
    end
    self:applySettings()
end

function core:applySettings()
    if self.db.profile.updatetime ~= oldTime then
        self:cancelTimer()
        self:setTimer()
        oldTime = self.db.profile.updatetime
    end
    if IsInGuild() then
        core:RegisterEvent("GUILDBANKFRAME_OPENED", "countGuild")
        core:RegisterEvent("GUILDBANKBAGSLOTS_CHANGED", "countGuild")
    end
    core:onTextUpdate()
end

function core:setTimer()
    self.updateTimer = self:ScheduleRepeatingTimer("countInv", self.db.profile.updatetime)
end

function core:cancelTimer()
    self:CancelTimer(self.updateTimer)
end

function core:combatRestart()
    self:onTextUpdate()
    self:setTimer()
end

function core:combatCancel()
    private.bem.text = ("|cffff0000"..L["Paused for Combat!"].."|r")
    self:cancelTimer()
end

function core:countGuild()
    if not IsInGuild() or private.scanPending then private.bem.text = ("|cffffff00"..L["Scan Pending"].."|r") return end
    private.scanPending = true
    private.bem.text = ("|cffffff00"..L["Scanning Guild Bank"].."|r")
    self.timerOne = self:ScheduleTimer("delayScan", self.db.profile.scantime)
end

function core:delayScan()
    for k,itemid in pairs (private.vanmats.id) do
        self:guildBank(itemid)
    end
    -- for k,itemid in pairs (private.bcmats.id) do
        -- self:guildBank(itemid)
    -- end
    -- for k,itemid in pairs (private.lkmats.id) do
        -- self:guildBank(itemid)
    -- end
    -- for k,itemid in pairs (private.catamats.id) do
        -- self:guildBank(itemid)
    -- end
    -- for k,itemid in pairs (private.mopsmats.id) do
        -- self:guildBank(itemid)
    -- end
    private.scanPending = false
    self:countInv()
end

core:RegisterEvent("PLAYER_REGEN_ENABLED", "combatRestart")
core:RegisterEvent("PLAYER_REGEN_DISABLED", "combatCancel")

function core:countInv()
    for k,itemid in pairs (private.vanmats.id) do
        self:countSave(itemid)
    end
    -- for k,itemid in pairs (private.bcmats.id) do
        -- self:countSave(itemid)
    -- end
    -- for k,itemid in pairs (private.lkmats.id) do
        -- self:countSave(itemid)
    -- end
    -- for k,itemid in pairs (private.catamats.id) do
        -- self:countSave(itemid)
    -- end
    -- for k,itemid in pairs (private.mopsmats.id) do
        -- self:countSave(itemid)
    -- end
    self:onTextUpdate()
end

function core:guildBank(itemid)
local total = 0
    local numTabs = GetNumGuildBankTabs()
    for tab = 1, numTabs do
        local name, icon, isViewable, canDeposit, numWithdrawals, remainingWithdrawals = GetGuildBankTabInfo(tab)
        if isViewable then
            for slot = 1, MAX_GUILDBANK_SLOTS_PER_TAB do
                local texture, count, locked = GetGuildBankItemInfo(tab, slot)
                if count > 0 and locked == nil then
                    local link = GetGuildBankItemLink(tab, slot)
                    if link ~= nil then
                    local id = link:match('|Hitem:(%d+):[%d:]+|h%[(.-)%]|h')
                    id = tonumber(id)
                        if id == itemid then
                            total = total + count
                        end
                    end
                end
            end
        end
    end
    self.db.realm.chars[self.charName].data[itemid].guild = total or 0
end

function core:countSave(itemid)
    local itemid = itemid
    local inv = GetItemCount(itemid)
    local bank = GetItemCount(itemid, true)
    bank = bank - inv
    self.db.realm.chars[self.charName].data[itemid].inv = inv or 0
    self.db.realm.chars[self.charName].data[itemid].bank = bank or 0
end

function core:CharHasElement(itemid)
    if self.db.realm.chars[self.charName].data[itemid].inv > 0 or (self.db.profile.showBank and self.db.realm.chars[self.charName].data[itemid].bank > 0) or (self.db.profile.showGuild and self.db.realm.chars[self.charName].data[itemid].guild > 0)then
        return true
    end
    if self.db.profile.showAlts then
        for char, _ in pairs(self.db.realm.chars) do
            if self.db.realm.chars[char].data[itemid].inv > 0 or (self.db.profile.showBank and self.db.realm.chars[char].data[itemid].bank > 0) or (self.db.profile.showGuild and self.db.realm.chars[char].data[itemid].guild > 0) then
                return true
            end
        end
    end
    return false
end

function core:CountString(itemid, char)
    if  core.db.profile.showZero and self.db.profile.showBank and self.db.profile.showGuild then
        return self:InvCountString(itemid, char)..self:BankCountString(itemid, char)..self:GuildCountString(itemid, char)
    elseif not core.db.profile.showZero and self.db.profile.showBank and self.db.profile.showGuild then
        return self:BankCountString(itemid, char)..self:GuildCountString(itemid, char)
    elseif core.db.profile.showZero and self.db.profile.showBank then
        return self:InvCountString(itemid, char)..self:BankCountString(itemid, char)
    elseif core.db.profile.showZero and self.db.profile.showGuild then
        return self:InvCountString(itemid, char)..self:GuildCountString(itemid, char)
    elseif core.db.profile.showZero then
        return self:InvCountString(itemid, char)
    elseif self.db.profile.showBank then
        return self:BankCountString(itemid, char)
    elseif self.db.profile.showGuild then
        return self:GuildCountString(itemid, char)
    end
end

function core:InvCountString(itemid, char)
    local tempChar = self.db.realm.chars[self.charName]
    if char then
        tempChar = self.db.realm.chars[char]
    end
    return self:Color(tempChar.data[itemid].inv)
end

function core:BankCountString(itemid, char)
    local tempChar = self.db.realm.chars[self.charName]
    if char then
        tempChar = self.db.realm.chars[char]
    end
    if self.db.profile.showBank then
        return "   "..self:Color(tempChar.data[itemid].bank)
    else
        return " "
    end
end

function core:GuildCountString(itemid, char)
    local tempChar = self.db.realm.chars[self.charName]
    if char then
        tempChar = self.db.realm.chars[char]
    end
    if self.db.profile.showGuild then
        return "   "..self:Color(tempChar.data[itemid].guild)
    else
        return " "
    end
end

function core:Color(count)
    local perc = 0
    local max = 10
    if max ~= 0 then
        perc = count / max
    end
    local r1, g1, b1
    local r2, g2, b2
    if perc <= 0.5 then
        perc = perc * 2
        r1, g1, b1 = 1, 0, 0
        r2, g2, b2 = 1, 1, 0
    else
        perc = perc * 2 - 1
        r1, g1, b1 = 1, 1, 0
        r2, g2, b2 = 0, 1, 0
    end
    local r, g, b = r1 + (r2 - r1)*perc, g1 + (g2 - g1)*perc, b1 + (b2 - b1)*perc
    if r < 0 then
        r = 0
    elseif r > 1 then
        r = 1
    end
    if g < 0 then
        g = 0
    elseif g > 1 then
        g = 1
    end
    if b < 0 then
        b = 0
    elseif b > 1 then
        b = 1
    end
    r, g, b =  r * 255, g * 255, b * 255
    color = strformat("%02x%02x%02x", r, g, b)
    count = ("|cff"..color..count.."|r")
    return count
end